package com.xhkjedu.smarking.listener;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @className MessageSender
 * @description
 * @date 2021/10/25 14:28
 **/
@Component
@Slf4j
public class MessageSender {
    @Autowired
    private AmqpTemplate rabbitTemplate;
    //负载一生产者mq
    @Value("${rabbitmq.markingImgMergeQueue}")
    private String markingImgMergeQueue;
    @Value("${rabbitmq.markingBaseToPdfQueue}")
    private String markingBaseToPdfQueue;
    @Value("${rabbitmq.markingBaseToScanImgQueue}")
    private String markingBaseToScanImgQueue;
    @Value("${rabbitmq.markingImgToPdfQueue}")
    private String markingImgToPdfQueue;
    @Value("${rabbitmq.markingHtmlToPdfQueue}")
    private String markingHtmlToPdfQueue;
    //负载二生产者mq
    @Value("${rabbitmq.markingImgMergeQueue2}")
    private String markingImgMergeQueue2;
    @Value("${rabbitmq.markingBaseToPdfQueue2}")
    private String markingBaseToPdfQueue2;
    @Value("${rabbitmq.markingBaseToScanImgQueue2}")
    private String markingBaseToScanImgQueue2;
    @Value("${rabbitmq.markingImgToPdfQueue2}")
    private String markingImgToPdfQueue2;
    //负载二节点名称
    private final String node2="node02/";

    /**
     * @Description 学生答案合并
     * @Param [epsqid, userAnswers, device]
     * @Return void
     * @Author wn
     * @Date 2022/7/28 16:46
     **/
    public void imgMerge(Integer epsqid, List<String> userAnswers, Integer device) {
        Map map = new HashMap();
        map.put("epsqid", epsqid);
        map.put("stuanswers", userAnswers);
        map.put("device", device);
        if (userAnswers.toString().contains(node2)) {
            rabbitTemplate.convertAndSend(markingImgMergeQueue2, JSON.toJSONString(map));
        } else {
            rabbitTemplate.convertAndSend(markingImgMergeQueue, JSON.toJSONString(map));
        }
    }

    /**
     * @Description 答题卡图片合并为pdf
     * @Param [epid, imgs]
     * @Return void
     * @Author wn
     * @Date 2022/8/2 13:57
     **/
    public void base64ToPdf(Integer markingid,Integer epid, List<String> imgs,String savefolder,Integer scantroncol,String filename) {
        Map map = new HashMap();
        map.put("markingid", markingid);
        map.put("epid",epid);
        map.put("imgs", imgs);
        map.put("scantroncol", scantroncol);
        map.put("savefolder",savefolder);
        map.put("filename",filename);
        if (imgs.toString().contains(node2)) {
            rabbitTemplate.convertAndSend(markingBaseToPdfQueue2, JSON.toJSONString(map));
        } else {
            rabbitTemplate.convertAndSend(markingBaseToPdfQueue, JSON.toJSONString(map));
        }
    }

    /**
     * @Description  扫码出错图片base64转图片
     * @Param [serrorid, imgs, savefolder]
     * @Return void
     * @Author wn
     * @Date 2022/9/19 9:40
     **/
    public void base64ToScanImg(Integer serrorid,Integer markingid,List<String> imgs,String savefolder) {
        Map map = new HashMap();
        map.put("serrorid", serrorid);
        map.put("markingid", markingid);
        map.put("imgs", imgs);
        map.put("savefolder",savefolder);
        if (imgs.toString().contains(node2)) {
            rabbitTemplate.convertAndSend(markingBaseToScanImgQueue2, JSON.toJSONString(map));
        } else {
            rabbitTemplate.convertAndSend(markingBaseToScanImgQueue, JSON.toJSONString(map));
        }
    }

    /**
     * @Description 考试附件试卷图片转pdf
     * @Param [imgs, epid]
     * @Return void
     * @Author wn
     * @Date 2022/11/1 10:43
     **/
    public void ImgToPdf(List<String> imgs,Integer epid) {
        Map map = new HashMap();
        map.put("epid",epid);
        map.put("imgs", imgs);
        if (imgs.toString().contains(node2)) {
            rabbitTemplate.convertAndSend(markingImgToPdfQueue2, JSON.toJSONString(map));
        } else {
            rabbitTemplate.convertAndSend(markingImgToPdfQueue, JSON.toJSONString(map));
        }
    }

    /**
     * @Description 考试报告html转pdf
     * @Param [map]
     * @Author ywx
     * @Date 2023/2/14 11:24
     * @return void
     **/
    public void htmlToPdf(Map map) {
        rabbitTemplate.convertAndSend(markingHtmlToPdfQueue, JSON.toJSONString(map));
    }
}
