package com.xhkjedu.smarking.service.paper;

import com.xhkjedu.smarking.mapper.exam.MsSubjectMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperBlockMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperBlockQuestionMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperMapper;
import com.xhkjedu.smarking.model.paper.MsPaper;
import com.xhkjedu.smarking.model.paper.MsPaperBlock;
import com.xhkjedu.smarking.model.paper.MsPaperBlockQuestion;
import com.xhkjedu.smarking.vo.paper.MsPaperParams;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description 阅卷试卷题块 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperBlockService {
    @Resource
    private MsPaperBlockMapper msPaperBlockMapper;
    @Resource
    private MsPaperMapper msPaperMapper;
    @Resource
    private MsPaperBlockQuestionMapper msPaperBlockQuestionMapper;
    @Resource
    private MsSubjectMapper msSubjectMapper;

    /*
     * @Description 获取试卷所有题块
     * @Date 2024/10/21 14:36:00
     * @Author WN
     * @Param [msid]
     * @Return java.util.List<com.xhkjedu.smarking.model.paper.MsPaperBlock>
     **/
    public List<MsPaperBlock> listPaperBlockQuestions(Integer msid){
        Integer blocktype = 2;
        return msPaperBlockMapper.listPaperBlockQuestions(msid,blocktype);
    }


    /**
     * @Description 新增
     * @Param [msPaperBlock]
     * @Author auto
     * @Date 2024-10-15
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public Integer save(MsPaperParams params){
        MsPaper paper = msPaperMapper.getPaperBaseByMsid(params.getMsid());
        Integer mpid = paper.getMpid();
        //先删除已存在题块
        msPaperBlockMapper.deleteObjByMpidAndBlocktype(mpid,2);

        Integer createid = paper.getCreateid();
        Integer timestamp = N_Utils.getSecondTimestamp();
        List<MsPaperBlock> blocks = params.getBlocks();
        for(MsPaperBlock block : blocks){
            block.setBlocktype(2);
            block.setMpid(mpid);
            block.setCreateid(createid);
            block.setCreatetime(timestamp);
        }
        msPaperBlockMapper.insertList(blocks);

        List<MsPaperBlockQuestion> questions = new ArrayList<>();
        for(MsPaperBlock block : blocks){
            List<MsPaperBlockQuestion> bqs = block.getQuestions();
            for(MsPaperBlockQuestion bq : bqs){
                bq.setMblockid(block.getId());
                bq.setMpid(mpid);
                bq.setCreateid(createid);
                bq.setCreatetime(timestamp);
                questions.add(bq);
            }
        }

        msPaperBlockQuestionMapper.insertList(questions);
        //更改科目题块框选状态
        msSubjectMapper.updatePbstateByMpid(mpid,1);//题块状态变成设置中
        //保存完题块后，需要清除之前已经生产的学生答案、批阅信息之类的数据，
        //知识点，难易度关联需要重新处理；新的题块试题需要绑定知识点，难度度如何处理？

        return mpid;
    }
}
