package com.xhkjedu.smarking.service.papercheck;

import com.xhkjedu.exception.ServiceException;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckMapper;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckTeacherMapper;
import com.xhkjedu.smarking.model.papercheck.MsPaperCheck;
import com.xhkjedu.smarking.model.papercheck.MsPaperCheckTeacher;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description 阅卷试卷阅卷方式表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperCheckService {
    @Resource
    private MsPaperCheckMapper msPaperCheckMapper;
    @Resource
    private MsPaperCheckTeacherMapper msPaperCheckTeacherMapper;

    /**
     * @Description 设置评卷方式
     * @Date 2024/10/24 14:16
     * @Author YWX
     * @Param [msPaperCheck]
     * @Return void
     **/
    public void add(MsPaperCheck msPaperCheck) {
        saveOrUpdate(msPaperCheck);
    }

    //添加或更新阅卷方式
    private MsPaperCheck saveOrUpdate(MsPaperCheck msPaperCheck) {
        Integer mpcid = msPaperCheckMapper.getMpcIdByMBlockId(msPaperCheck.getMblockid());
        if (N_Utils.isTrueInteger(mpcid)) {
            msPaperCheck.setMpcid(mpcid);
            msPaperCheckMapper.updateByPrimaryKeySelective(msPaperCheck);
        } else {
            msPaperCheck.setCreatetime(N_Utils.getSecondTimestamp());
            msPaperCheckMapper.insertSelective(msPaperCheck);
        }
        return msPaperCheck;
    }

    /**
     * @Description 阅卷分配列表
     * @Date 2024/10/24 14:19
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listPaperCheck(Integer mpid) {
        return msPaperCheckMapper.listPaperCheck(mpid);
    }

    /**
     * @Description 设置仲裁教师
     * @Date 2024/10/24 14:40
     * @Author YWX
     * @Param [msPaperCheck]
     * @Return void
     **/
    public void setArbitrate(MsPaperCheck msPaperCheck) {
        saveOrUpdate(msPaperCheck);
    }

    /**
     * @Description 设置阅卷教师
     * @Date 2024/10/24 15:24
     * @Author YWX
     * @Param [msPaperCheck]
     * @Return void
     **/
    public void setCheckTeacher(MsPaperCheck msPaperCheck) {
        MsPaperCheck check = saveOrUpdate(msPaperCheck);
        Integer mpid = check.getMpid();
        Integer mblockid = check.getMblockid();
        Integer createid = msPaperCheck.getCreateid();
        int timestamp = N_Utils.getSecondTimestamp();
        List<MsPaperCheckTeacher> teachers = msPaperCheck.getTeachers();
        for (MsPaperCheckTeacher teacher : teachers) {
            teacher.setMpid(mpid);
            teacher.setMblockid(mblockid);
            teacher.setCreateid(createid);
            teacher.setCreatetime(timestamp);
        }
        msPaperCheckTeacherMapper.insertList(teachers);
    }

    /**
     * @Description 设置分配方式
     * @Date 2024/10/24 15:41
     * @Author YWX
     * @Param [msPaperCheck]
     * @Return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void setDispenseWay(MsPaperCheck msPaperCheck) {
        Integer dispenseway = msPaperCheck.getDispenseway();
        if (dispenseway.equals(3)) {
            List<MsPaperCheckTeacher> teachers = msPaperCheck.getTeachers();
            if (N_Utils.isListEmpty(teachers)) throw new ServiceException("请选择阅卷教师");
            for (MsPaperCheckTeacher teacher : teachers) {
                if (N_Utils.isTrueInteger(teacher.getTasknum())) {
                    throw new ServiceException("请设置每位教师的阅卷任务数");
                }
            }
            msPaperCheckTeacherMapper.batchSave(teachers);
        }
        saveOrUpdate(msPaperCheck);
    }
}
